import { POST_PATH, POST_ROUTE_PREFIX } from '@blog/config';
import { readdir } from 'node:fs/promises';
import type { Post } from '@blog/models/Post';
import * as yamlFront from 'yaml-front-matter';
import { marked } from 'marked';

export async function readPostFile(path: string): Promise<Post> {
  const file = Bun.file(path);
  const fileContent = await file.text();
  const parsedData = yamlFront.loadFront(fileContent);  
  const postHtml = await marked.parse(parsedData.__content);

  let slug = parsedData.slug;

  if (!slug) {
    slug = path.split('/').pop().slice(0, -3);
  }

  return {
    meta: {
      title: parsedData.title,
      description: parsedData.description,
      date: new Date(parsedData.date),
      draft: parsedData?.draft || false,
      tags: parsedData.tags,
      slug: slug
    },
    content: parsedData.__content,
    html: postHtml
  };
}

export class PostFileService {
  private posts: Map<string, Post>; 

  public constructor(posts: Map<string, Post>) {
    this.posts = posts;
  }

  public static async create() {
    const posts = new Map<string, Post>();
    const postFiles: string[] = await readdir(POST_PATH);

    for (const postFile of postFiles) {
      const post = await readPostFile(POST_PATH + "/" + postFile);
      const key = post.meta?.slug || postFile.slice(0, -3);
      posts.set(key, post);
    }

    return new PostFileService(posts);
  }

  public getPosts(): Map<string, Post> {
    return this.posts;
  }

  public getPost(slug: string): Post {
    const post = this.posts.get(slug);

    if (!post) {
      throw Error("Post does not exist.");
    }

    return post;
  }

  public getPostHtml(slug: string): string {
    const html = this.getPost(slug).html;

    if (!html) {
      throw Error("No HTML for this post yet.");
    }

    return html;
  }

  public getPostRawContent(slug: string): string {
    return this.getPost(slug).content;
  }
}
